package inf43.spring2009.console;

import inf43.spring2009.console.converters.Converter;
import inf43.spring2009.console.validators.Validator;
import java.util.ArrayList;
import java.util.List;


public class Prompt
{
	public static String read(String promptText, String helpText)
	{
		return read(
			promptText, helpText,
			new Converter<String>()
			{
				public String convert(String input)
				{
					return input;
				}
			});
	}
	
	
	public static <ResultType> ResultType read(
		String promptText, String helpText, Converter<ResultType> converter)
	{
		return read(
			promptText, helpText, converter,
			new ArrayList<Validator<ResultType>>());
	}
	
	
	public static <ResultType> ResultType read(
		String promptText, String helpText,
		Converter<ResultType> converter,
		Validator<ResultType> validator)
	{
		List<Validator<ResultType>> validators = new ArrayList<Validator<ResultType>>();
		validators.add(validator);
		
		return read(promptText, helpText, converter, validators);
	}
	
	
	public static <ResultType> ResultType read(
		String promptText, String helpText, Converter<ResultType> converter,
		List<? extends Validator<ResultType>> validators)
	{
		String input;
		
		do
		{
			Console.write(promptText + ": ");

			input = Console.readLine();
			
			if (input.trim().equals("?"))
			{
				Console.writeLine();
				Console.writeLine(helpText);
				Console.writeLine();
			}
		}
		while (input.trim().equals("?"));
		
		ResultType result = converter.convert(input);
		
		for (Validator<ResultType> validator : validators)
		{
			validator.validate(result);
		}
		
		return result;
	}
}
