package inf43.spring2009.console.comparators;

import java.util.Comparator;
import java.util.List;


public class ComparatorCombiner
{
	public static <E> Comparator<E> combine(List<Comparator<E>> comparators)
	{
		if (comparators == null)
		{
			return buildNullComparator();
		}
		else
		{
			return buildComparator(comparators, 0);
		}
	}


	private static <E> Comparator<E> buildComparator(
		final List<Comparator<E>> comparators, final int index)
	{
		if (index >= comparators.size())
		{
			return buildNullComparator();
		}
		else
		{
			final Comparator<E> nextComparator = buildComparator(comparators, index + 1);
			
			return new Comparator<E>()
			{
				public int compare(E e1, E e2)
				{
					int comparison = comparators.get(index).compare(e1, e2);
					return comparison == 0 ? nextComparator.compare(e1, e2) : comparison;
				}
			};
		}
	}
	
	
	private static <E> Comparator<E> buildNullComparator()
	{
		return new Comparator<E>()
		{
			public int compare(E e1, E e2)
			{
				return 0;
			}
		};
	}
}
