package inf43.spring2009.console.menus;

import inf43.spring2009.console.Console;
import inf43.spring2009.console.converters.ConversionException;
import inf43.spring2009.console.validators.ValidationException;
import java.util.ArrayList;
import java.util.List;


public class Menu<ModelType> implements CommandHandler<ModelType>
{
	private String title;
	private List<Command<ModelType>> commands;
	
	
	public Menu(String title, List<Command<ModelType>> commands)
	{
		this.title = title;
		this.commands = new ArrayList<Command<ModelType>>(commands);
	}
	
	
	public CommandHandlerResult run(ModelType model)
	{
		show();
		
		while (true)
		{
			Console.writeLine();
			Console.write("[" + title + "] Command: ");
			String commandString = Console.readLine().trim().toUpperCase();
			System.out.println();

			CommandHandlerResult result = null;
			
			for (Command<ModelType> command : commands)
			{
				if (command.getCommandString().toUpperCase().equals(commandString))
				{
					try
					{
						result = command.handle(model);
					}
					catch (CommandFailedException e)
					{
						Console.writeLine(e.getMessage());
						result = CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
					}
					catch (ConversionException e)
					{
						Console.writeLine(e.getMessage());
						result = CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
					}
					catch (ValidationException e)
					{
						Console.writeLine(e.getMessage());
						result = CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
					}
					catch (Exception e)
					{
						Console.writeLine("Unexpected failure");
						Console.writeLine(e.getClass().getName());
						Console.writeLine(e.getMessage());
						result = CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
					}

					break;
				}
			}
			
			if (result == null)
			{
				if (commandString.equals("?"))
				{
					show();
				}
				else
				{
					Console.writeLine("Unrecognized command: " + commandString);
				}
			}
			else if (result == CommandHandlerResult.NO_MORE_COMMANDS)
			{
				return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
			}
		}
	}
	
	
	private void show()
	{
		Console.writeLine(title);
		
		for (int i = 0; i < title.length(); i++)
		{
			Console.write("-");
		}

		Console.writeLine();
		
		for (Command<ModelType> command : commands)
		{
			Console.writeLine("[" + command.getCommandString() + "] " + command.getDescription());
		}
	}
}
