package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.Console;
import inf43.spring2009.console.Prompt;
import inf43.spring2009.console.converters.ToIntConverter;
import inf43.spring2009.console.converters.TrimStringConverter;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.console.validators.IntIsPositiveValidator;
import inf43.spring2009.console.validators.StringIsNotEmptyValidator;
import inf43.spring2009.tpes.model.Classroom;
import inf43.spring2009.tpes.model.DuplicateClassroomNameException;
import inf43.spring2009.tpes.model.TpesData;


public class AddClassroomCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
	throws CommandFailedException
	{
		String name = Prompt.read(
			"Name",
			"Enter the classroom name",
			new TrimStringConverter(),
			new StringIsNotEmptyValidator("A classroom name must be specified"));
		
		int capacity = Prompt.read(
			"Capacity",
			"Enter the capacity of the classrom, which must be at least 1",
			new ToIntConverter(),
			new IntIsPositiveValidator("A positive number must be specified"));

		try
		{
			model.getClassrooms().addClassroom(new Classroom(name, capacity));
		}
		catch (DuplicateClassroomNameException e)
		{
			throw new CommandFailedException(
				"That classroom name already exists; classroom names must be unique");
		}

		Console.writeLine("Added classroom: " + name);
		
		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
}
