package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.Console;
import inf43.spring2009.console.Prompt;
import inf43.spring2009.console.converters.ToIntConverter;
import inf43.spring2009.console.converters.TrimStringConverter;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.console.validators.IntIsNonNegativeValidator;
import inf43.spring2009.console.validators.IntIsPositiveValidator;
import inf43.spring2009.console.validators.StringIsNotEmptyValidator;
import inf43.spring2009.console.validators.StringMatchesRegexValidator;
import inf43.spring2009.tpes.model.Course;
import inf43.spring2009.tpes.model.DuplicateCourseIdException;
import inf43.spring2009.tpes.model.DuplicateCourseNameException;
import inf43.spring2009.tpes.model.TpesData;


public class AddCourseCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
		throws CommandFailedException
	{
		String id = Prompt.read(
			"ID",
			"Enter the course ID, which must be a four-digit number",
			new TrimStringConverter(),
			new StringMatchesRegexValidator(
				"\\d{4}",
				"Course IDs must consist of exactly four digits (0-9)"));

		String name = Prompt.read(
			"Name",
			"Enter the course name",
			new TrimStringConverter(),
			new StringIsNotEmptyValidator("A course name must be specified"));

		int maximumCapacity = Prompt.read(
			"Maximum Capacity",
			"Enter the maximum capacity of the course, which must be at least 1",
			new ToIntConverter(),
			new IntIsPositiveValidator("Maximum capacity must be a positive number"));

		int minimumNumberOfMinutes = Prompt.read(
			"Minimum Minutes Per Week",
			"Enter the minimum minutes per week that the course should meet, which must be at least 0",
			new ToIntConverter(),
			new IntIsNonNegativeValidator("Minimum number of minutes per week must be at least 0"));

		try
		{
			model.getCourses().addCourse(
				new Course(id, name, maximumCapacity, minimumNumberOfMinutes));
		}
		catch (DuplicateCourseIdException e)
		{
			throw new CommandFailedException(
				"A course with that ID already exists; course IDs must be unique");
		}
		catch (DuplicateCourseNameException e)
		{
			throw new CommandFailedException(
				"A course with that name already exists; course names must be unique");
		}

		Console.writeLine("Added Course: " + name);

		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
}
