package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.Console;
import inf43.spring2009.console.Prompt;
import inf43.spring2009.console.converters.TrimStringConverter;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.console.validators.StringIsNotEmptyValidator;
import inf43.spring2009.console.validators.StringMatchesRegexValidator;
import inf43.spring2009.tpes.model.DuplicateStudentIdException;
import inf43.spring2009.tpes.model.Student;
import inf43.spring2009.tpes.model.TpesData;


public class AddStudentCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
	throws CommandFailedException
	{
		String id = Prompt.read(
			"ID",
			"Enter the student ID, which must be a seven-digit number",
			new TrimStringConverter(),
			new StringMatchesRegexValidator("\\d{7}", "IDs must be exactly seven digits (0-9)"));
		
		String lastName = Prompt.read(
			"Last Name",
			"Enter the student's last name",
			new TrimStringConverter(),
			new StringIsNotEmptyValidator("Last name must be specified"));
		
		String firstNameEtc = Prompt.read(
			"First/Middle Name",
			"Enter the student's first/middle name (e.g., Joe Paul); the middle name is not required",
			new TrimStringConverter(),
			new StringIsNotEmptyValidator("First/middle name must be specified"));

		try
		{
			model.getStudents().addStudent(new Student(id, lastName, firstNameEtc));
		}
		catch (DuplicateStudentIdException e)
		{
			throw new CommandFailedException(
				"A student with that ID already exists; student IDs must be unique");
		}

		Console.writeLine("Student Added: " + lastName + ", " + firstNameEtc);
		
		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
}
