package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.Console;
import inf43.spring2009.console.Prompt;
import inf43.spring2009.console.converters.TrimStringConverter;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.console.validators.StringIsNotEmptyValidator;
import inf43.spring2009.tpes.formatters.StudentFullNameFormatter;
import inf43.spring2009.tpes.matchers.CourseByIdMatcher;
import inf43.spring2009.tpes.matchers.StudentByIdMatcher;
import inf43.spring2009.tpes.model.Course;
import inf43.spring2009.tpes.model.NoSuchCourseException;
import inf43.spring2009.tpes.model.NoSuchStudentException;
import inf43.spring2009.tpes.model.NotEnrolledException;
import inf43.spring2009.tpes.model.Student;
import inf43.spring2009.tpes.model.TpesData;


public class DropStudentFromCourseCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
	throws CommandFailedException
	{
		String studentId = Prompt.read(
			"Student ID",
			"Enter the ID of the student you'd like to drop from a course",
			new TrimStringConverter(),
			new StringIsNotEmptyValidator("The student ID is required"));
		
		String courseId = Prompt.read(
			"Course ID",
			"Enter the ID of the course you'd like to drop that student from",
			new TrimStringConverter(),
			new StringIsNotEmptyValidator("The course ID is required"));

		Student student;
		Course course;

		try
		{
			student = model.getStudents().findStudent(
				new StudentByIdMatcher(studentId));
		}
		catch (NoSuchStudentException e)
		{
			throw new CommandFailedException("There is no student with that ID");
		}

		try
		{
			course = model.getCourses().findCourse(
				new CourseByIdMatcher(courseId));
		}
		catch (NoSuchCourseException e)
		{
			throw new CommandFailedException("There is no course with that ID");
		}

		try
		{
			student.dropCourse(course);
			course.dropStudent(student);
		}
		catch (NotEnrolledException e)
		{
			throw new CommandFailedException("That student is not enrolled in that course");
		}
		
		Console.writeLine(
			new StudentFullNameFormatter().format(student)
			+ " has been dropped from " + course.getName());
		
		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
}
