package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.Console;
import inf43.spring2009.console.Prompt;
import inf43.spring2009.console.converters.TrimStringConverter;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.tpes.formatters.MeetingIntervalFormatter;
import inf43.spring2009.tpes.formatters.StudentFullNameFormatter;
import inf43.spring2009.tpes.matchers.CourseByIdMatcher;
import inf43.spring2009.tpes.matchers.StudentByIdMatcher;
import inf43.spring2009.tpes.model.AlreadyEnrolledException;
import inf43.spring2009.tpes.model.Course;
import inf43.spring2009.tpes.model.CourseFullException;
import inf43.spring2009.tpes.model.CourseMeetingOverlapsException;
import inf43.spring2009.tpes.model.NoSuchCourseException;
import inf43.spring2009.tpes.model.NoSuchStudentException;
import inf43.spring2009.tpes.model.NotEnrolledException;
import inf43.spring2009.tpes.model.Student;
import inf43.spring2009.tpes.model.TpesData;
import inf43.spring2009.tpes.validators.CourseIdExistsValidator;
import inf43.spring2009.tpes.validators.StudentIdExistsValidator;


public class EnrollStudentIntoCourseCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
	throws CommandFailedException
	{
		String studentId = Prompt.read(
			"Student ID", "Enter the ID of the student you'd like to enroll",
			new TrimStringConverter(),
			new StudentIdExistsValidator(model, "There is no student with that ID"));

		String courseId = Prompt.read(
			"Course ID", "Enter the ID of the course you'd like to enroll the student into",
			new TrimStringConverter(),
			new CourseIdExistsValidator(model, "There is no course with that ID"));

		Student student;
		Course course;
		
		try
		{
			student = model.getStudents().findStudent(
				new StudentByIdMatcher(studentId));
		}
		catch (NoSuchStudentException e)
		{
			throw new CommandFailedException("There is no student with that ID");
		}

		try
		{
			course = model.getCourses().findCourse(
				new CourseByIdMatcher(courseId));
		}
		catch (NoSuchCourseException e)
		{
			throw new CommandFailedException("There is no course with that ID");
		}
		
		try
		{
			student.enrollInCourse(course);
		}
		catch (AlreadyEnrolledException e)
		{
			throw new CommandFailedException(
				"That student is already enrolled in that course");
		}
		catch (CourseMeetingOverlapsException e)
		{
			throw new CommandFailedException(
				"Cannot enroll student in " + course.getName()
				+ " since it meets on "
				+ new MeetingIntervalFormatter().format(e.getNewMeeting().getMeetingInterval())
				+ Console.getLineSeparator()
				+ "but the student is already enrolled in " + e.getExistingMeeting().getCourse().getName()
				+ " which meets on "
				+ new MeetingIntervalFormatter().format(e.getExistingMeeting().getMeetingInterval()));
		}
		
		try
		{
			course.enrollStudent(student);
		}
		catch (CourseFullException e)
		{
			try
			{
				student.dropCourse(course);
			}
			catch (NotEnrolledException ee)
			{
			}
			
			throw new CommandFailedException("That course is full");
		}
		
		Console.writeLine(
			new StudentFullNameFormatter().format(student)
			+ " is now enrolled in " + course.getName());
		
		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
}
