package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.displays.TableColumnDisplay;
import inf43.spring2009.console.displays.TableDisplay;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.tpes.comparators.CoursesByNameComparator;
import inf43.spring2009.tpes.formatters.SchedulingStatusFormatter;
import inf43.spring2009.tpes.model.Course;
import inf43.spring2009.tpes.model.TpesData;
import java.util.ArrayList;
import java.util.List;


public class ListCoursesCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
	throws CommandFailedException
	{
		new TableDisplay<Course>().show(
			"Courses", "There are no courses",
			model.getCourses(),
			new CoursesByNameComparator(),
			createColumnDisplays());
		
		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
	
	
	private List<TableColumnDisplay<Course>> createColumnDisplays()
	{
		List<TableColumnDisplay<Course>> displays = new ArrayList<TableColumnDisplay<Course>>();
		
		displays.add(
			new TableColumnDisplay<Course>("ID", 4)
			{
				protected String toDisplayString(Course course)
				{
					return course.getId();
				}
			});
		
		displays.add(
			new TableColumnDisplay<Course>("Name", 30)
			{
				protected String toDisplayString(Course course)
				{
					return course.getName();
				}
			});
		
		displays.add(
			new TableColumnDisplay<Course>("MaxCap", 6)
			{
				protected String toDisplayString(Course course)
 				{
					return "" + course.getMaximumCapacity();
				}
			});
		
		displays.add(
			new TableColumnDisplay<Course>("EffCap", 6)
			{
				protected String toDisplayString(Course course)
				{
					return "" + course.getEffectiveCapacity();
				}
			});
		
		displays.add(
			new TableColumnDisplay<Course>("#Enrolled", 8)
			{
				protected String toDisplayString(Course course)
				{
					return "" + course.getEnrolledStudents().size();
				}
			});
		
		displays.add(
			new TableColumnDisplay<Course>("MinTime", 7)
			{
				protected String toDisplayString(Course course)
				{
					return "" + course.getMinimumNumberOfMinutes();
				}
			});
		
		displays.add(
			new TableColumnDisplay<Course>("SchTime", 7)
			{
				protected String toDisplayString(Course course)
				{
					return "" + course.getScheduledNumberOfMinutes();
				}
			});
		
		displays.add(
			new TableColumnDisplay<Course>("Scheduling Status", 19)
			{
				protected String toDisplayString(Course course)
				{
					return new SchedulingStatusFormatter().format(course.getSchedulingStatus());
				}
			});
		
		return displays;
	}
}
