package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.comparators.ComparatorCombiner;
import inf43.spring2009.console.displays.TableColumnDisplay;
import inf43.spring2009.console.displays.TableDisplay;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.tpes.comparators.StudentsByFirstNameEtcComparator;
import inf43.spring2009.tpes.comparators.StudentsByIdComparator;
import inf43.spring2009.tpes.comparators.StudentsByLastNameComparator;
import inf43.spring2009.tpes.formatters.StudentFullNameFormatter;
import inf43.spring2009.tpes.model.Student;
import inf43.spring2009.tpes.model.TpesData;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;


public class ListStudentsCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
	throws CommandFailedException
	{
		new TableDisplay<Student>().show(
			"Students", "There are no students",
			model.getStudents(),
			ComparatorCombiner.combine(createComparators()),
			createTableColumnDisplays());
		
		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
	
	
	private static List<Comparator<Student>> createComparators()
	{
		List<Comparator<Student>> students = new ArrayList<Comparator<Student>>();
		
		students.add(new StudentsByLastNameComparator());
		students.add(new StudentsByFirstNameEtcComparator());
		students.add(new StudentsByIdComparator());
		
		return students;
	}
	
	
	private static List<TableColumnDisplay<Student>> createTableColumnDisplays()
	{
		List<TableColumnDisplay<Student>> students = new ArrayList<TableColumnDisplay<Student>>();
		
		students.add(
			new TableColumnDisplay<Student>("ID", 7)
			{
				protected String toDisplayString(Student student)
				{
					return student.getId();
				}
			});
		
		students.add(
			new TableColumnDisplay<Student>("Name", 50)
			{
				protected String toDisplayString(Student student)
				{
					return new StudentFullNameFormatter().format(student);
				}
			});
		
		return students;
	}
}
