package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.Console;
import inf43.spring2009.console.Prompt;
import inf43.spring2009.console.converters.TrimStringConverter;
import inf43.spring2009.console.displays.DetailsDisplay;
import inf43.spring2009.console.displays.DetailsFieldDisplay;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.console.validators.StringIsNotEmptyValidator;
import inf43.spring2009.tpes.comparators.CourseMeetingsByDayAndTimeComparator;
import inf43.spring2009.tpes.formatters.MeetingIntervalFormatter;
import inf43.spring2009.tpes.matchers.ClassroomByNameMatcher;
import inf43.spring2009.tpes.model.Classroom;
import inf43.spring2009.tpes.model.CourseMeeting;
import inf43.spring2009.tpes.model.NoSuchClassroomException;
import inf43.spring2009.tpes.model.TpesData;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


public class ViewClassroomInformationCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
	throws CommandFailedException
	{
		String name = Prompt.read(
			"Name",
			"Enter the name of the classrom whose information you'd like to view",
			new TrimStringConverter(),
			new StringIsNotEmptyValidator("A classroom name is required"));

		Console.writeLine();
		
		try
		{
			Classroom classroom =
				model.getClassrooms()
				.findClassroom(new ClassroomByNameMatcher(name));
			
			DetailsDisplay<Classroom> classroomDetails =
				new DetailsDisplay<Classroom>(createFieldDisplays());
			
			classroomDetails.show(classroom);
		}
		catch (NoSuchClassroomException e)
		{
			Console.writeLine("There is no classroom with that name");
		}
		
		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
	
	
	private List<DetailsFieldDisplay<Classroom>> createFieldDisplays()
	{
		ArrayList<DetailsFieldDisplay<Classroom>> fieldDisplays =
			new ArrayList<DetailsFieldDisplay<Classroom>>();
		
		fieldDisplays.add(
			new DetailsFieldDisplay<Classroom>("Name")
			{
				protected String toDisplayString(Classroom classroom)
				{
					return classroom.getName();
				}
			});
		
		fieldDisplays.add(
			new DetailsFieldDisplay<Classroom>("Capacity")
			{
				protected String toDisplayString(Classroom classroom)
				{
					return "" + classroom.getCapacity();
				}
			});
		
		fieldDisplays.add(
			new DetailsFieldDisplay<Classroom>("Course Meetings")
			{
				protected String toDisplayString(Classroom classroom)
				{
					if (classroom.getCourseMeetings().size() == 0)
					{
						return "None";
					}
					
					List<CourseMeeting> meetings =
						new ArrayList<CourseMeeting>(classroom.getCourseMeetings());
					
					Collections.sort(meetings, new CourseMeetingsByDayAndTimeComparator());

					String displayString = "";
					
					for (CourseMeeting meeting : meetings)
					{
						if (displayString.length() > 0)
						{
							displayString += Console.getLineSeparator();
						}
						
						displayString +=
							new MeetingIntervalFormatter().format(meeting.getMeetingInterval())
							+ " " + meeting.getCourse().getName();
					}
					
					return displayString;
				}
			});
		
		return fieldDisplays;
	}
}
