package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.Console;
import inf43.spring2009.console.Prompt;
import inf43.spring2009.console.converters.TrimStringConverter;
import inf43.spring2009.console.displays.DetailsDisplay;
import inf43.spring2009.console.displays.DetailsFieldDisplay;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.console.validators.StringIsNotEmptyValidator;
import inf43.spring2009.tpes.formatters.MeetingIntervalFormatter;
import inf43.spring2009.tpes.formatters.SchedulingStatusFormatter;
import inf43.spring2009.tpes.formatters.StudentFullNameFormatter;
import inf43.spring2009.tpes.matchers.CourseByIdMatcher;
import inf43.spring2009.tpes.model.Course;
import inf43.spring2009.tpes.model.CourseMeeting;
import inf43.spring2009.tpes.model.NoSuchCourseException;
import inf43.spring2009.tpes.model.Student;
import inf43.spring2009.tpes.model.TpesData;
import java.util.ArrayList;
import java.util.List;


public class ViewCourseInformationCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
	throws CommandFailedException
	{
		String courseId = Prompt.read(
			"ID",
			"Enter the ID of the course for which you'd like to view information",
			new TrimStringConverter(),
			new StringIsNotEmptyValidator("The ID must be specified"));
		
		Console.writeLine();
		
		try
		{
			Course course = model.getCourses().findCourse(
				new CourseByIdMatcher(courseId));
			
			DetailsDisplay<Course> display =
				new DetailsDisplay<Course>(createFieldDisplays());
			
			display.show(course);
		}
		catch (NoSuchCourseException e)
		{
			Console.writeLine("There is no course with that ID");
		}
		
		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
	
	
	private List<DetailsFieldDisplay<Course>> createFieldDisplays()
	{
		List<DetailsFieldDisplay<Course>> displays = new ArrayList<DetailsFieldDisplay<Course>>();
		
		displays.add(
			new DetailsFieldDisplay<Course>("ID")
			{
				protected String toDisplayString(Course course)
				{
					return course.getId();
				}
			});
		
		displays.add(
			new DetailsFieldDisplay<Course>("Name")
			{
				protected String toDisplayString(Course course)
				{
					return course.getName();
				}
			});
		
		displays.add(
			new DetailsFieldDisplay<Course>("Maximum Capacity")
			{
				protected String toDisplayString(Course course)
				{
					return "" + course.getMaximumCapacity();
				}
			});
		
		displays.add(
			new DetailsFieldDisplay<Course>("Effective Capacity")
			{
				protected String toDisplayString(Course course)
				{
					return "" + course.getEffectiveCapacity();
				}
			});
		
		displays.add(
			new DetailsFieldDisplay<Course>("Minimum Minutes")
			{
				protected String toDisplayString(Course course)
				{
					return "" + course.getMinimumNumberOfMinutes();
				}
			});
		
		displays.add(
			new DetailsFieldDisplay<Course>("Scheduled Minutes")
			{
				protected String toDisplayString(Course course)
				{
					return "" + course.getScheduledNumberOfMinutes();
				}
			});
		
		displays.add(
			new DetailsFieldDisplay<Course>("Scheduling Status")
			{
				protected String toDisplayString(Course course)
				{
					return new SchedulingStatusFormatter().format(course.getSchedulingStatus());
				}
			});
		
		displays.add(
			new DetailsFieldDisplay<Course>("Scheduled Meetings")
			{
				protected String toDisplayString(Course course)
				{
					List<CourseMeeting> meetings = course.getCourseMeetings();
					
					if (meetings.size() == 0)
					{
						return "None";
					}
					else
					{
						String displayString = "";
						
						for (CourseMeeting meeting : meetings)
						{
							if (displayString.length() > 0)
							{
								displayString += Console.getLineSeparator();
							}
							
							displayString += new MeetingIntervalFormatter().format(
								meeting.getMeetingInterval());
						}
						
						return displayString;
					}
				}
			});
		
		displays.add(
			new DetailsFieldDisplay<Course>("Enrolled Students")
			{
				protected String toDisplayString(Course course)
				{
					List<Student> enrolledStudents = course.getEnrolledStudents();
					
					if (enrolledStudents.size() == 0)
					{
						return "None";
					}
					else
					{
						StringBuilder builder = new StringBuilder();
						
						for (Student student : enrolledStudents)
						{
							if (builder.length() > 0)
							{
								builder.append(Console.getLineSeparator());
							}
							
							builder.append(
								student.getId() + "  "
								+ new StudentFullNameFormatter().format(student));
						}
						
						return builder.toString();
					}
				}
			});
		
		return displays;
	}
}
