package inf43.spring2009.tpes.commands;

import inf43.spring2009.console.Console;
import inf43.spring2009.console.Prompt;
import inf43.spring2009.console.converters.TrimStringConverter;
import inf43.spring2009.console.displays.DetailsDisplay;
import inf43.spring2009.console.displays.DetailsFieldDisplay;
import inf43.spring2009.console.menus.CommandFailedException;
import inf43.spring2009.console.menus.CommandHandler;
import inf43.spring2009.console.menus.CommandHandlerResult;
import inf43.spring2009.console.validators.StringIsNotEmptyValidator;
import inf43.spring2009.tpes.formatters.StudentFullNameFormatter;
import inf43.spring2009.tpes.matchers.StudentByIdMatcher;
import inf43.spring2009.tpes.model.Course;
import inf43.spring2009.tpes.model.NoSuchStudentException;
import inf43.spring2009.tpes.model.Student;
import inf43.spring2009.tpes.model.TpesData;
import java.util.ArrayList;
import java.util.List;


public class ViewStudentInformationCommandHandler implements CommandHandler<TpesData>
{
	public CommandHandlerResult run(TpesData model)
	throws CommandFailedException
	{
		String studentId = Prompt.read(
			"ID",
			"Enter the ID of the student whose information you'd like to view",
			new TrimStringConverter(),
			new StringIsNotEmptyValidator("The ID must be specified"));
		
		Console.writeLine();
		
		try
		{
			Student student = model.getStudents().findStudent(
				new StudentByIdMatcher(studentId));
			
			DetailsDisplay<Student> display =
				new DetailsDisplay<Student>(createFieldDisplays());
			
			display.show(student);
		}
		catch (NoSuchStudentException e)
		{
			Console.writeLine("There is no such student");
		}
		
		return CommandHandlerResult.ACCEPT_ANOTHER_COMMAND;
	}
	
	
	private List<DetailsFieldDisplay<Student>> createFieldDisplays()
	{
		List<DetailsFieldDisplay<Student>> fieldDisplays =
			new ArrayList<DetailsFieldDisplay<Student>>();
		
		fieldDisplays.add(
			new DetailsFieldDisplay<Student>("ID")
			{
				protected String toDisplayString(Student student)
				{
					return student.getId();
				}
			});
		
		fieldDisplays.add(
			new DetailsFieldDisplay<Student>("Name")
			{
				protected String toDisplayString(Student student)
				{
					return new StudentFullNameFormatter().format(student);
				}
			});
		
		fieldDisplays.add(
			new DetailsFieldDisplay<Student>("Enrolled Courses")
			{
				protected String toDisplayString(Student student)
				{
					List<Course> enrolledCourses = student.getEnrolledCourses();
					
					if (enrolledCourses.size() == 0)
					{
						return "None";
					}
					else
					{
						String courses = "";
						
						for (Course enrolledCourse : student.getEnrolledCourses())
						{
							if (courses.length() > 0)
							{
								courses += Console.getLineSeparator();
							}
							
							courses += enrolledCourse.getName();
						}
						
						return courses;
					}
				}
			});
		
		return fieldDisplays;
	}
}
