package inf43.spring2009.tpes.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


public class Classroom implements Serializable
{
	private String name;
	private int capacity;
	private List<CourseMeeting> courseMeetings;
	
	
	public Classroom(String name, int capacity)
	{
		setName(name);
		setCapacity(capacity);
		courseMeetings = new ArrayList<CourseMeeting>();
	}
	
	
	public String getName()
	{
		return name;
	}
	
	
	public void setName(String name)
	{
		if (name == null || name.trim().length() == 0)
		{
			throw new IllegalArgumentException("name must be non-empty");
		}
		
		this.name = name.trim();
	}
	
	
	public int getCapacity()
	{
		return capacity;
	}
	
	
	public void setCapacity(int capacity)
	{
		if (capacity <= 0)
		{
			throw new IllegalArgumentException("capacity must be positive");
		}
		
		this.capacity = capacity;
	}
	
	
	public List<CourseMeeting> getCourseMeetings()
	{
		return Collections.unmodifiableList(courseMeetings);
	}
	
	
	public void addCourseMeeting(CourseMeeting courseMeetingToAdd)
	throws CourseMeetingOverlapsException
	{
		if (courseMeetingToAdd.getClassroom() != this)
		{
			throw new IllegalArgumentException(
				"courseMeetingToAdd refers to the wrong classroom");
		}
		
		for (CourseMeeting courseMeeting : courseMeetings)
		{
			if (courseMeeting.getMeetingInterval().overlaps(courseMeetingToAdd.getMeetingInterval()))
			{
				throw new CourseMeetingOverlapsException(courseMeeting, courseMeetingToAdd);
			}
		}
		
		courseMeetings.add(courseMeetingToAdd);
	}
	
	
	public void removeCourseMeeting(CourseMeeting courseMeeting)
	{
		if (courseMeetings.contains(courseMeeting))
		{
			courseMeetings.remove(courseMeeting);
		}
	}
	
	
	public void clearCourseMeetings()
	{
		courseMeetings.clear();
	}
}
