package inf43.spring2009.tpes.model;

import inf43.spring2009.console.matchers.Matcher;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


public class ClassroomCollection implements Iterable<Classroom>, Serializable
{
	private List<Classroom> classrooms;
	
	
	public ClassroomCollection()
	{
		classrooms = new ArrayList<Classroom>();
	}
	
	
	public void addClassroom(Classroom classroomToAdd)
	throws DuplicateClassroomNameException
	{
		for (Classroom classroom : classrooms)
		{
			if (classroom.getName().equals(classroomToAdd.getName()))
			{
				throw new DuplicateClassroomNameException();
			}
		}
		
		classrooms.add(classroomToAdd);
	}
	
	
	public void removeClassroom(Matcher<Classroom> classroomMatcher)
	throws NoSuchClassroomException
	{
		Classroom classroomToRemove = findClassroom(classroomMatcher);

		for (CourseMeeting courseMeeting : classroomToRemove.getCourseMeetings())
		{
			courseMeeting.getCourse().removeCourseMeeting(courseMeeting);
		}
		
		classrooms.remove(classroomToRemove);
	}
	
	
	public Classroom findClassroom(Matcher<Classroom> classroomMatcher)
	throws NoSuchClassroomException
	{
		for (Classroom classroom : classrooms)
		{
			if (classroomMatcher.matches(classroom))
			{
				return classroom;
			}
		}
		
		throw new NoSuchClassroomException();
	}
	
	
	public List<Classroom> findClassrooms(Matcher<Classroom> classroomMatcher)
	{
		List<Classroom> matchedClassrooms = new ArrayList<Classroom>();
		
		for (Classroom classroom : classrooms)
		{
			if (classroomMatcher.matches(classroom))
			{
				matchedClassrooms.add(classroom);
			}
		}
		
		return matchedClassrooms;
	}
	
	
	public void clearAllCourseMeetings()
	{
		for (Classroom classroom : classrooms)
		{
			classroom.clearCourseMeetings();
		}
	}
	
	
	public Iterator<Classroom> iterator()
	{
		return classrooms.iterator();
	}
}
