package inf43.spring2009.tpes.model;

import inf43.spring2009.console.matchers.Matcher;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


public class CourseCollection implements Iterable<Course>, Serializable
{
	private List<Course> courses;
	
	
	public CourseCollection()
	{
		courses = new ArrayList<Course>();
	}

	
	public void addCourse(Course courseToAdd)
	throws DuplicateCourseIdException, DuplicateCourseNameException
	{
		for (Course course : courses)
		{
			if (course.getId().equals(courseToAdd.getId()))
			{
				throw new DuplicateCourseIdException();
			}
			
			if (course.getName().equals(courseToAdd.getName()))
			{
				throw new DuplicateCourseNameException();
			}
		}
		
		courses.add(courseToAdd);
	}
	
	
	public void removeCourse(Matcher<Course> courseMatcher)
	throws NoSuchCourseException
	{
		Course courseToRemove = findCourse(courseMatcher);
		
		for (CourseMeeting courseMeeting : courseToRemove.getCourseMeetings())
		{
			courseMeeting.getClassroom().removeCourseMeeting(courseMeeting);
		}
		
		for (Student student : courseToRemove.getEnrolledStudents())
		{
			try
			{
				student.dropCourse(courseToRemove);
			}
			catch (NotEnrolledException e)
			{
			}
		}
		
		courses.remove(courseToRemove);
	}
	
	
	public Course findCourse(Matcher<Course> courseMatcher)
	throws NoSuchCourseException
	{
		for (Course course : courses)
		{
			if (courseMatcher.matches(course))
			{
				return course;
			}
		}
		
		throw new NoSuchCourseException();
	}
	
	
	public List<Course> findCourses(Matcher<Course> courseMatcher)
	{
		List<Course> matchingCourses = new ArrayList<Course>();
		
		for (Course course : courses)
		{
			if (courseMatcher.matches(course))
			{
				matchingCourses.add(course);
			}
		}
		
		return matchingCourses;
	}
	
	
	public void clear()
	{
		courses.clear();
	}
	
	
	public Iterator<Course> iterator()
	{
		return courses.iterator();
	}
}
