package inf43.spring2009.tpes.model;

import java.io.Serializable;


public class MeetingInterval implements Serializable
{
	private DayOfWeek dayOfWeek;
	private TimeOfDay startTime;
	private TimeOfDay endTime;
	
	
	public MeetingInterval(DayOfWeek dayOfWeek, TimeOfDay startTime, TimeOfDay endTime)
	{
		if (endTime.before(startTime))
		{
			throw new IllegalArgumentException("endTime must be equal to or after startTime");
		}
		
		this.dayOfWeek = dayOfWeek;
		this.startTime = startTime;
		this.endTime = endTime;
	}
	
	
	public DayOfWeek getDayOfWeek()
	{
		return dayOfWeek;
	}
	
	
	public TimeOfDay getStartTime()
	{
		return startTime;
	}
	
	
	public TimeOfDay getEndTime()
	{
		return endTime;
	}
	
	
	public int getLengthInMinutes()
	{
		return endTime.difference(startTime);
	}
	
	
	public boolean overlaps(MeetingInterval other)
	{
		if (dayOfWeek == other.getDayOfWeek())
		{
			if (other.getStartTime().before(startTime))
			{
				return other.getEndTime().after(startTime);
			}
			else if (other.getStartTime().before(endTime))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		else
		{
			return false;
		}
	}
}
