
package inf43.spring2009.tpes.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


public class Student implements Serializable
{
	private String id;
	private String lastName;
	private String firstNameEtc;
	private List<Course> enrolledCourses;
	
	
	public Student(String id, String lastName, String firstNameEtc)
	{
		setId(id);
		setLastName(lastName);
		setFirstNameEtc(firstNameEtc);
		enrolledCourses = new ArrayList<Course>();
	}
	
	
	public String getId()
	{
		return id;
	}
	
	
	private void setId(String id)
	{
		if (id == null || id.length() != 7 ||
			!Character.isDigit(id.charAt(0)) ||
			!Character.isDigit(id.charAt(1)) ||
			!Character.isDigit(id.charAt(2)) ||
			!Character.isDigit(id.charAt(3)) ||
			!Character.isDigit(id.charAt(4)) ||
			!Character.isDigit(id.charAt(5)) ||
			!Character.isDigit(id.charAt(6)))
		{
			throw new IllegalArgumentException("id must be a seven-digit number");
		}
		
		this.id = id;
	}
	
	
	public String getLastName()
	{
		return lastName;
	}
	
	
	private void setLastName(String lastName)
	{
		if (lastName == null || lastName.trim().length() == 0)
		{
			throw new IllegalArgumentException("lastName must be non-empty");
		}
		
		this.lastName = lastName.trim();
	}
	
	
	public String getFirstNameEtc()
	{
		return firstNameEtc;
	}
	
	
	private void setFirstNameEtc(String firstNameEtc)
	{
		if (firstNameEtc == null || firstNameEtc.trim().length() == 0)
		{
			throw new IllegalArgumentException("firstNameEtc must be non-empty");
		}
		
		this.firstNameEtc = firstNameEtc;
	}
	
	
	public List<Course> getEnrolledCourses()
	{
		return Collections.unmodifiableList(enrolledCourses);
	}
	
	
	public void enrollInCourse(Course courseToEnrollIn)
	throws AlreadyEnrolledException, CourseMeetingOverlapsException
	{
		if (courseToEnrollIn == null)
		{
			throw new IllegalArgumentException("courseToEnrollIn must not be null");
		}
		
		if (enrolledCourses.contains(courseToEnrollIn))
		{
			throw new AlreadyEnrolledException();
		}
		
		for (Course enrolledCourse : enrolledCourses)
		{
			for (CourseMeeting enrolledCourseMeeting : enrolledCourse.getCourseMeetings())
			{
				for (CourseMeeting courseMeetingToEnrollIn : courseToEnrollIn.getCourseMeetings())
				{
					if (enrolledCourseMeeting.getMeetingInterval().overlaps(
						courseMeetingToEnrollIn.getMeetingInterval()))
					{
						throw new CourseMeetingOverlapsException(
							enrolledCourseMeeting, courseMeetingToEnrollIn);
					}
				}
			}
		}
		
		enrolledCourses.add(courseToEnrollIn);
	}
	
	
	public void dropCourse(Course courseToDrop)
	throws NotEnrolledException
	{
		if (courseToDrop == null)
		{
			throw new IllegalArgumentException("courseToDrop must not be null");
		}
		
		if (enrolledCourses.contains(courseToDrop))
		{
			enrolledCourses.remove(courseToDrop);
		}
		else
		{
			throw new NotEnrolledException();
		}
	}
	
	
	public void clearEnrolledCourses()
	{
		enrolledCourses.clear();
	}
}
