package inf43.spring2009.tpes.model;

import inf43.spring2009.console.matchers.Matcher;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


public class StudentCollection implements Iterable<Student>, Serializable
{
	private List<Student> students;
	
	
	public StudentCollection()
	{
		students = new ArrayList<Student>();
	}
	
	
	public void addStudent(Student studentToAdd)
	throws DuplicateStudentIdException
	{
		for (Student student : students)
		{
			if (student.getId().equals(studentToAdd.getId()))
			{
				throw new DuplicateStudentIdException();
			}
		}
		
		students.add(studentToAdd);
	}
	
	
	public void removeStudent(Matcher<Student> studentMatcher)
	throws NoSuchStudentException
	{
		Student studentToRemove = findStudent(studentMatcher);
		students.remove(studentToRemove);
	}
	
	
	public Student findStudent(Matcher<Student> studentMatcher)
	throws NoSuchStudentException
	{
		for (Student student : students)
		{
			if (studentMatcher.matches(student))
			{
				return student;
			}
		}
		
		throw new NoSuchStudentException();
	}
	
	
	public List<Student> findStudents(Matcher<Student> studentMatcher)
	{
		List<Student> matchingStudents = new ArrayList<Student>();
		
		for (Student student : students)
		{
			if (studentMatcher.matches(student))
			{
				matchingStudents.add(student);
			}
		}
		
		return matchingStudents;
	}
	
	
	public void clearAllEnrolledCourses()
	{
		for (Student student : students)
		{
			student.clearEnrolledCourses();
		}
	}
	
	
	public Iterator<Student> iterator()
	{
		return students.iterator();
	}
}
