package inf43.spring2009.tpes.model;

import java.io.Serializable;


public class TimeOfDay implements Serializable, Comparable<TimeOfDay>
{
	private int hour;
	private int minute;
	private int minutesSinceMidnight;
	
	
	public TimeOfDay(int hour, int minute)
	{
		setHour(hour);
		setMinute(minute);
	}
	
	
	public int getHour()
	{
		return hour;
	}
	
	
	public void setHour(int hour)
	{
		if (hour < 0 || hour > 23)
		{
			throw new IllegalArgumentException("invalid hour: " + hour + "; hour must be between 0 and 23");
		}
		
		this.hour = hour;
		updateMinutesSinceMidnight();
	}
	
	
	public int getMinute()
	{
		return minute;
	}
	
	
	public void setMinute(int minute)
	{
		if (minute < 0 || minute > 59)
		{
			throw new IllegalArgumentException("invalid minute: " + minute + "; minute must be between 0 and 59");
		}
		
		this.minute = minute;
		updateMinutesSinceMidnight();
	}
	
	
	private void updateMinutesSinceMidnight()
	{
		minutesSinceMidnight = hour * 60 + minute;
	}
	
	
	public boolean equals(Object x)
	{
		if (x instanceof TimeOfDay)
		{
			TimeOfDay other = (TimeOfDay) x;
			return minutesSinceMidnight == other.minutesSinceMidnight;
		}
		else
		{
			return false;
		}
	}
	
	
	public boolean before(TimeOfDay other)
	{
		return minutesSinceMidnight < other.minutesSinceMidnight;
	}
	
	
	public boolean after(TimeOfDay other)
	{
		return minutesSinceMidnight > other.minutesSinceMidnight;
	}
	
	
	public int difference(TimeOfDay other)
	{
		return minutesSinceMidnight - other.minutesSinceMidnight;
	}


	public int compareTo(TimeOfDay other)
	{
		return difference(other);
	}
}
